@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2024 Hans Petter Jansson
+/* Copyright (C) 2018-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -66,8 +66,21 @@ init_features (void)
         have_sse41 = TRUE;
 # endif
 
+    /* For popcnt, AMD does not appear to need the SSE 4.2 check, but the Intel
+     * documentation[1] says the following in section 12.12.3:
+     *
+     * > Before an application attempts to use the POPCNT instruction, it
+     * > must check that the processor supports Intel SSE4.2 (if
+     * > CPUID.01H:ECX.SSE4_2[bit 20] = 1) and POPCNT (if
+     * > CPUID.01H:ECX.POPCNT[bit 23] = 1).
+     *
+     * [1] https://software.intel.com/en-us/download/intel-64-and-ia-32-architectures-sdm-combined-volumes-1-2a-2b-2c-2d-3a-3b-3c-3d-and-4
+     *
+     * So we check both. */
+
 # ifdef HAVE_POPCNT_INTRINSICS
-    if (__builtin_cpu_supports ("popcnt"))
+    if (__builtin_cpu_supports ("sse4.2")
+        && __builtin_cpu_supports ("popcnt"))
         have_popcnt = TRUE;
 # endif
 
@@ -231,7 +244,7 @@ chafa_set_n_threads (gint n)
 {
     g_return_if_fail (n >= -1);
 
-    return g_atomic_int_set (&n_threads, n);
+    g_atomic_int_set (&n_threads, n);
 }
 
 /**
@@ -245,13 +258,13 @@ chafa_set_n_threads (gint n)
 gint
 chafa_get_n_actual_threads (void)
 {
-    gint n_threads;
+    gint n_actual_threads;
 
-    n_threads = chafa_get_n_threads ();
-    if (n_threads < 0)
-        n_threads = g_get_num_processors ();
-    if (n_threads <= 0)
-        n_threads = 1;
+    n_actual_threads = chafa_get_n_threads ();
+    if (n_actual_threads < 0)
+        n_actual_threads = g_get_num_processors ();
+    if (n_actual_threads <= 0)
+        n_actual_threads = 1;
 
-    return n_threads;
+    return n_actual_threads;
 }
