@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2024 Hans Petter Jansson
+/* Copyright (C) 2018-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -20,6 +20,7 @@
 #include "config.h"
 #include "chafa.h"
 #include "internal/chafa-private.h"
+#include <stdint.h>
 
 /**
  * SECTION:chafa-frame
@@ -126,11 +127,13 @@ chafa_frame_new_steal (gpointer data,
  * Since: 1.14
  **/
 ChafaFrame *
-chafa_frame_new_borrow (gpointer data,
+chafa_frame_new_borrow (gconstpointer data,
                         ChafaPixelType pixel_type,
                         gint width, gint height, gint rowstride)
 {
-    return new_frame (data, pixel_type, width, height, rowstride, FALSE);
+    /* Suppress the warning we get with -Wcast-qual. We honor the constness
+     * by tracking it with the data_is_owned flag. */
+    return new_frame ((gpointer)(uintptr_t) data, pixel_type, width, height, rowstride, FALSE);
 }
 
 /**
