@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2024 Hans Petter Jansson
+/* Copyright (C) 2019-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -25,11 +25,60 @@
 #include "chafa.h"
 #include "internal/chafa-batch.h"
 
+static gint chafa_batch_n_threads_global;
+
+static gint
+allocate_threads (gint max_threads, gint n_batches)
+{
+    gint prev_n_threads;
+    gint n_threads;
+
+    /* GThreadPool supports sharing threads between pools, but there is no
+     * mechanism to manage the global thread count. If the batch API is being
+     * called from multiple threads, we risk creating N * M workers, which
+     * can result in hundreds of threads.
+     *
+     * Therefore, we maintain a global count of active threads and allocate
+     * each caller's allotment from that. The minimum allocation is 1 thread,
+     * in which case the operation is performed in the calling thread. Single-
+     * threaded tasks are allowed to overshoot the maximum, so maximum
+     * concurrency will be N + M - 1, where N is the number of calling threads
+     * and M is the requisition from chafa_get_n_actual_threads(). For typical
+     * workloads, average concurrency will likely be close to M. */
+
+    prev_n_threads = 0;
+    n_threads = MIN (max_threads, n_batches);
+
+    /* Geometric backoff: The first iteration adds to the global, subsequent
+     * iterations subtract until we're happy. */
+
+    for (;;)
+    {
+        gint next_n_global = n_threads
+            + g_atomic_int_add (&chafa_batch_n_threads_global,
+                                n_threads - prev_n_threads);
+        if (next_n_global <= max_threads || n_threads == 1)
+            break;
+
+        prev_n_threads = n_threads;
+        n_threads /= 2;
+    }
+
+    return n_threads;
+}
+
+static void
+deallocate_threads (gint n_threads)
+{
+    g_atomic_int_add (&chafa_batch_n_threads_global, -n_threads);
+}
+
 void
 chafa_process_batches (gpointer ctx, GFunc batch_func, GFunc post_func, gint n_rows, gint n_batches, gint batch_unit)
 {
     GThreadPool *thread_pool = NULL;
     ChafaBatchInfo *batches;
+    gint max_threads;
     gint n_threads;
     gint n_units;
     gfloat units_per_batch;
@@ -42,18 +91,22 @@ chafa_process_batches (gpointer ctx, GFunc batch_func, GFunc post_func, gint n_r
     if (n_rows < 1)
         return;
 
-    n_threads = MIN (chafa_get_n_actual_threads (), n_batches);
+    max_threads = chafa_get_n_actual_threads ();
+    n_threads = allocate_threads (max_threads, n_batches);
+
     n_units = (n_rows + batch_unit - 1) / batch_unit;
     units_per_batch = (gfloat) n_units / (gfloat) n_batches;
 
     batches = g_new0 (ChafaBatchInfo, n_batches);
 
     if (n_threads >= 2)
+    {
         thread_pool = g_thread_pool_new (batch_func,
                                          (gpointer) ctx,
                                          n_threads,
                                          FALSE,
                                          NULL);
+    }
 
     /* Divide work up into batches that are multiples of batch_unit, except
      * for the last one (if n_rows is not itself a multiple) */
@@ -124,4 +177,5 @@ chafa_process_batches (gpointer ctx, GFunc batch_func, GFunc post_func, gint n_r
     }
 
     g_free (batches);
+    deallocate_threads (n_threads);
 }
