@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2024 Hans Petter Jansson
+/* Copyright (C) 2019-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -461,7 +461,9 @@ chafa_kitty_canvas_build_ansi (ChafaKittyCanvas *kitty_canvas,
     {
         /* Make IDs in the first <256 range predictable, but as the range
          * cycles we add one to skip over every ID==0 */
-        if (placement_id > 255)
+        if (placement_id < 1)
+            placement_id = 1;
+        else if (placement_id > 255)
             placement_id = 1 + (placement_id % 255);
 
         build_unicode_virtual (kitty_canvas, term_info, out_str,
