@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2021-2024 Hans Petter Jansson
+/* Copyright (C) 2021-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -24,6 +24,8 @@
 
 G_BEGIN_DECLS
 
+#define CHAFA_SQUARE(n) ((n) * (n))
+
 void chafa_tuck_and_align (gint src_width, gint src_height,
                            gint dest_width, gint dest_height,
                            ChafaAlign halign, ChafaAlign valign,
@@ -31,6 +33,8 @@ void chafa_tuck_and_align (gint src_width, gint src_height,
                            gint *ofs_x_out, gint *ofs_y_out,
                            gint *width_out, gint *height_out);
 
+gint chafa_round_up_to_multiple_of (gint value, gint m);
+
 G_END_DECLS
 
 #endif /* __CHAFA_MATH_UTIL_H__ */
