@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2024 Hans Petter Jansson
+/* Copyright (C) 2018-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -24,10 +24,9 @@
 #include "internal/chafa-private.h"
 
 void
-calc_colors_mmx (const ChafaPixel *pixels, ChafaColorAccum *accums_out, const guint8 *cov)
+chafa_extract_cell_mean_colors_mmx (const ChafaPixel *pixels, ChafaColorAccum *accums_out, const guint8 *cov)
 {
     __m64 accum [2] = { 0 };
-    const guint32 *u32p0 = (const guint32 *) pixels;
     __m64 m64b;
     gint i;
 
@@ -39,7 +38,7 @@ calc_colors_mmx (const ChafaPixel *pixels, ChafaColorAccum *accums_out, const gu
         __m64 m64a;
 
         m64p1 = &accum [cov [i]];
-        m64a = _mm_cvtsi32_si64 (u32p0 [i]);
+        m64a = _mm_cvtsi32_si64 (chafa_color8_to_u32 (pixels [i].col));
         m64a = _mm_unpacklo_pi8 (m64a, m64b);
         *m64p1 = _mm_adds_pi16 (*m64p1, m64a);
     }
@@ -47,17 +46,5 @@ calc_colors_mmx (const ChafaPixel *pixels, ChafaColorAccum *accums_out, const gu
     ((__m64 *) accums_out) [0] = accum [0];
     ((__m64 *) accums_out) [1] = accum [1];
 
-#if 0
-    /* Called after outer loop is done */
     _mm_empty ();
-#endif
-}
-
-void
-chafa_leave_mmx (void)
-{
-#ifdef HAVE_MMX_INTRINSICS
-    if (chafa_have_mmx ())
-        _mm_empty ();
-#endif
 }
