@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2024 Hans Petter Jansson
+/* Copyright (C) 2019-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -23,7 +23,7 @@
 #include "internal/chafa-pca.h"
 
 #define PCA_POWER_MAX_ITERATIONS 1000
-#define PCA_POWER_MIN_ERROR 0.0001
+#define PCA_POWER_MIN_ERROR 0.0001f
 
 static gfloat
 pca_converge (const ChafaVec3f32 *vecs_in, gint n_vecs,
