@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2020-2024 Hans Petter Jansson
+/* Copyright (C) 2020-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -37,7 +37,12 @@ void chafa_prepare_pixel_data_for_symbols (const ChafaPalette *palette,
                                            gint src_rowstride,
                                            ChafaPixel *dest_pixels,
                                            gint dest_width,
-                                           gint dest_height);
+                                           gint dest_height,
+                                           gint cell_width,
+                                           gint cell_height,
+                                           ChafaAlign halign,
+                                           ChafaAlign valign,
+                                           ChafaTuck tuck);
 
 void chafa_sort_pixel_index_by_channel (guint8 *index,
                                         const ChafaPixel *pixels, gint n_pixels,
