@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2024 Hans Petter Jansson
+/* Copyright (C) 2018-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -27,6 +27,8 @@
 #include "internal/chafa-indexed-image.h"
 #include "internal/chafa-iterm2-canvas.h"
 #include "internal/chafa-kitty-canvas.h"
+#include "internal/chafa-math-util.h"
+#include "internal/chafa-noise.h"
 #include "internal/chafa-palette.h"
 #include "internal/chafa-sixel-canvas.h"
 
@@ -34,7 +36,7 @@ G_BEGIN_DECLS
 
 /* Character symbols and symbol classes */
 
-#define CHAFA_N_SYMBOLS_MAX 1024  /* For static temp arrays */
+#define CHAFA_N_SYMBOLS_MAX 2048  /* For static temp arrays */
 #define CHAFA_SYMBOL_N_PIXELS (CHAFA_SYMBOL_WIDTH_PIXELS * CHAFA_SYMBOL_HEIGHT_PIXELS)
 
 typedef struct
@@ -199,26 +201,23 @@ void chafa_canvas_config_init (ChafaCanvasConfig *canvas_config);
 void chafa_canvas_config_deinit (ChafaCanvasConfig *canvas_config);
 void chafa_canvas_config_copy_contents (ChafaCanvasConfig *dest, const ChafaCanvasConfig *src);
 
-gint *chafa_gen_bayer_matrix (gint matrix_size, gdouble magnitude);
+gint *chafa_gen_bayer_matrix (gint matrix_size, gfloat magnitude);
 
 /* Math stuff */
 
 #ifdef HAVE_MMX_INTRINSICS
-void calc_colors_mmx (const ChafaPixel *pixels, ChafaColorAccum *accums_out, const guint8 *cov);
-void chafa_leave_mmx (void);
-#else
-# define chafa_leave_mmx()
+void chafa_extract_cell_mean_colors_mmx (const ChafaPixel *pixels, ChafaColorAccum *accums_out, const guint8 *cov);
 #endif
 
 #ifdef HAVE_SSE41_INTRINSICS
-gint calc_error_sse41 (const ChafaPixel *pixels, const ChafaColorPair *color_pair, const guint8 *cov) G_GNUC_PURE;
+gint chafa_calc_cell_error_sse41 (const ChafaPixel *pixels, const ChafaColorPair *color_pair, const guint8 *cov);
 #endif
 
 #ifdef HAVE_AVX2_INTRINSICS
-gint calc_error_avx2 (const ChafaPixel *pixels, const ChafaColorPair *color_pair,
-                      const guint32 *sym_mask_u32) G_GNUC_PURE;
-void calc_colors_avx2 (const ChafaPixel *pixels, ChafaColorAccum *accums_out,
-                       const guint32 *sym_mask_u32);
+gint chafa_calc_cell_error_avx2 (const ChafaPixel *pixels, const ChafaColorPair *color_pair,
+                                 const guint32 *sym_mask_u32);
+void chafa_extract_cell_mean_colors_avx2 (const ChafaPixel *pixels, ChafaColorAccum *accums_out,
+                                          const guint32 *sym_mask_u32);
 void chafa_color_accum_div_scalar_avx2 (ChafaColorAccum *accum, guint16 divisor);
 #endif
 
@@ -239,6 +238,7 @@ static inline guint64 chafa_slow_pop_count (guint64 v) G_GNUC_UNUSED;
 static inline gint chafa_population_count_u64 (guint64 v) G_GNUC_UNUSED;
 static inline void chafa_population_count_vu64 (const guint64 *vv, gint *vc, gint n) G_GNUC_UNUSED;
 
+__attribute__ ((no_sanitize ("integer")))
 static inline guint64
 chafa_slow_pop_count (guint64 v)
 {
