@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2019-2024 Hans Petter Jansson
+/* Copyright (C) 2019-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -24,6 +24,7 @@
 #include "internal/chafa-batch.h"
 #include "internal/chafa-bitfield.h"
 #include "internal/chafa-indexed-image.h"
+#include "internal/chafa-math-util.h"
 #include "internal/chafa-passthrough-encoder.h"
 #include "internal/chafa-sixel-canvas.h"
 #include "internal/chafa-string-util.h"
@@ -51,13 +52,6 @@ typedef struct
 }
 SixelRow;
 
-static gint
-round_up_to_multiple_of (gint value, gint m)
-{
-    value = value + m - 1;
-    return value - (value % m);
-}
-
 ChafaSixelCanvas *
 chafa_sixel_canvas_new (gint width, gint height,
                         ChafaColorSpace color_space,
@@ -70,7 +64,7 @@ chafa_sixel_canvas_new (gint width, gint height,
     sixel_canvas->width = width;
     sixel_canvas->height = height;
     sixel_canvas->color_space = color_space;
-    sixel_canvas->image = chafa_indexed_image_new (width, round_up_to_multiple_of (height, SIXEL_CELL_HEIGHT),
+    sixel_canvas->image = chafa_indexed_image_new (width, chafa_round_up_to_multiple_of (height, SIXEL_CELL_HEIGHT),
                                                    palette, dither);
 
     if (!sixel_canvas->image)
@@ -94,7 +88,8 @@ chafa_sixel_canvas_draw_all_pixels (ChafaSixelCanvas *sixel_canvas, ChafaPixelTy
                                     gconstpointer src_pixels,
                                     gint src_width, gint src_height, gint src_rowstride,
                                     ChafaAlign halign, ChafaAlign valign,
-                                    ChafaTuck tuck)
+                                    ChafaTuck tuck,
+                                    gfloat quality)
 {
     g_return_if_fail (sixel_canvas != NULL);
     g_return_if_fail (src_pixel_type < CHAFA_PIXEL_MAX);
@@ -112,7 +107,8 @@ chafa_sixel_canvas_draw_all_pixels (ChafaSixelCanvas *sixel_canvas, ChafaPixelTy
                                      src_width, src_height, src_rowstride,
                                      sixel_canvas->width, sixel_canvas->height,
                                      halign, valign,
-                                     tuck);
+                                     tuck,
+                                     quality);
 }
 
 #define FILTER_BANK_WIDTH 64
@@ -505,8 +501,8 @@ chafa_sixel_canvas_build_ansi (ChafaSixelCanvas *sixel_canvas, ChafaTermInfo *te
     g_snprintf (buf,
                 CHAFA_TERM_SEQ_LENGTH_MAX,
                 "\"1;1;%d;%d",
-                sixel_canvas->image->width,
-                sixel_canvas->image->height);
+                sixel_canvas->width,
+                sixel_canvas->height);
     chafa_passthrough_encoder_append (&ptenc, buf);
 
     ctx.sixel_canvas = sixel_canvas;
