@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-/* Copyright (C) 2018-2024 Hans Petter Jansson
+/* Copyright (C) 2018-2025 Hans Petter Jansson
  *
  * This file is part of Chafa, a program that shows pictures on text terminals.
  *
@@ -51,23 +51,20 @@ fetch_canvas_pixel_block (const ChafaPixel *src_image, gint src_width,
 
     for ( ; row_p < end_p; row_p += src_width)
     {
-        const ChafaPixel *p0 = row_p;
-        const ChafaPixel *p1 = p0 + CHAFA_SYMBOL_WIDTH_PIXELS;
-
-        for ( ; p0 < p1; p0++)
-            pixels_out [i++] = *p0;
+        memcpy (&pixels_out [i], row_p, CHAFA_SYMBOL_WIDTH_PIXELS * sizeof (ChafaPixel));
+        i += CHAFA_SYMBOL_WIDTH_PIXELS;
     }
 }
 
 static void
-calc_colors_plain (const ChafaPixel *block, ChafaColorAccum *accums, const guint8 *cov)
+extract_cell_mean_colors_plain (const ChafaPixel *block, ChafaColorAccum *accums, const guint8 *cov)
 {
     const guint8 *in_u8 = (const guint8 *) block;
     gint i;
 
     for (i = 0; i < CHAFA_SYMBOL_N_PIXELS; i++)
     {
-        gint16 *out_s16 = (gint16 *) (accums + *cov++);
+        gint16 *out_s16 = accums [*cov++].ch;
 
         *out_s16++ += *in_u8++;
         *out_s16++ += *in_u8++;
@@ -85,14 +82,14 @@ chafa_work_cell_get_mean_colors_for_symbol (const ChafaWorkCell *wcell, const Ch
 
 #ifdef HAVE_AVX2_INTRINSICS
     if (chafa_have_avx2 ())
-        calc_colors_avx2 (wcell->pixels, accums, sym->mask_u32);
+        chafa_extract_cell_mean_colors_avx2 (wcell->pixels, accums, sym->mask_u32);
     else
 #elif defined(HAVE_MMX_INTRINSICS)
     if (chafa_have_mmx ())
-        calc_colors_mmx (wcell->pixels, accums, covp);
+        chafa_extract_cell_mean_colors_mmx (wcell->pixels, accums, covp);
     else
 #endif
-        calc_colors_plain (wcell->pixels, accums, covp);
+        extract_cell_mean_colors_plain (wcell->pixels, accums, covp);
 
     if (sym->fg_weight > 1)
         chafa_color_accum_div_scalar (&accums [1], sym->fg_weight);
