@@ -808,7 +808,7 @@ find_repacks (const SmolImplementation **implementations,
               const SmolRepackMeta **src_repack, const SmolRepackMeta **dest_repack)
 {
     int src_impl, dest_impl;
-    const SmolRepackMeta *src_meta, *dest_meta = NULL;
+    const SmolRepackMeta *src_meta = NULL, *dest_meta = NULL;
     uint16_t src_to_mid_sig, mid_to_dest_sig;
     uint16_t sig_mask;
     int reorder_dest_alpha_ch;
@@ -858,7 +858,7 @@ find_repacks (const SmolImplementation **implementations,
                     do_reorder (mid_order, dest_order,
                                 reorder_meta [SMOL_REPACK_SIGNATURE_GET_REORDER (dest_meta->signature)].dest);
 
-                    if (*((uint32_t *) dest_order) == *((uint32_t *) dest_pmeta->order))
+                    if (*((const uint32_t *) dest_order) == *((const uint32_t *) dest_pmeta->order))
                     {
                         /* Success */
                         goto out;
@@ -1254,7 +1254,7 @@ smol_scale_new_simple (const void *src_pixels,
 {
     SmolScaleCtx *scale_ctx;
 
-    scale_ctx = calloc (sizeof (SmolScaleCtx), 1);
+    scale_ctx = calloc (1,sizeof (SmolScaleCtx));
     smol_scale_init (scale_ctx,
                      src_pixels,
                      src_pixel_type,
@@ -1354,7 +1354,7 @@ smol_scale_new_full (const void *src_pixels,
 {
     SmolScaleCtx *scale_ctx;
 
-    scale_ctx = calloc (sizeof (SmolScaleCtx), 1);
+    scale_ctx = calloc (1, sizeof (SmolScaleCtx));
     smol_scale_init (scale_ctx,
                      src_pixels,
                      src_pixel_type,
